=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-07-23 21:41:49 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-08-16 21:53:52 +0000
@@ -62,6 +62,21 @@
             path = h.get_path(revid, file_id)
         filename = os.path.basename(path)
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files'))
+
+        # Create breadcrumb trail for the path within the branch
+        try:
+            inv = h.get_inventory(revid)
+        except:
+            self.log.exception('Exception fetching changes')
+            raise HTTPServerError('Could not fetch changes')
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
+
         return {
             'revid': revid,
             'file_id': file_id,
@@ -71,4 +86,6 @@
             'change': h.get_changes([ revid ])[0],
             'contents': list(h.annotate_file(file_id, revid)),
             'fileview_active': True,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'branch_breadcrumbs': branch_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/changelog_ui.py	2008-08-16 21:53:52 +0000
@@ -86,6 +86,13 @@
                     all_same_author = False
                     break
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'changes': changes,
@@ -100,4 +107,5 @@
             'search_failed': search_failed,
             'all_same_author': all_same_author,
             'url': self._branch.context_url,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-08-07 04:39:09 +0000
+++ loggerhead/controllers/directory_ui.py	2008-08-16 21:53:52 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import branch
 
+from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 
 class DirEntry(object):
@@ -71,16 +72,13 @@
             dirs.append(DirEntry(d, parity, b))
             parity = 1 - parity
         # Create breadcrumb trail
-        dir_parts = self._name.strip('/').split('/')
-        breadcrumbs = []
-        for index, dir_name in enumerate(dir_parts):
-            breadcrumbs.append({
-                'dir_name': dir_name,
-                'path': '/'.join(dir_parts[:index + 1]),
-            })
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._name,
+                False,
+                'directory')
         return {
             'dirs': dirs,
             'name': self._name,
             'static_url': static_url,
-            'breadcrumbs': breadcrumbs,
+            'directory_breadcrumbs': directory_breadcrumbs,
             }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-08-12 23:10:24 +0000
+++ loggerhead/controllers/inventory_ui.py	2008-08-16 21:53:52 +0000
@@ -75,36 +75,15 @@
         if updir == '/':
             updir_file_id = None
 
-        # Is our root directory itself a branch?
-        if self._branch.is_root:
-            outer_breadcrumbs = []
-            root_name = self._branch.friendly_name
-            root_suffix = 'files'
-        else:
-            # Create breadcrumb trail for the path leading up to the branch
-            outer_breadcrumbs = []
-            dir_parts = self._branch.friendly_name.strip('/').split('/')
-            for index, dir_name in enumerate(dir_parts):
-                outer_breadcrumbs.append({
-                    'dir_name': dir_name,
-                    'path': '/'.join(dir_parts[:index + 1]),
-                    'suffix': '',
-                })
-            # The branch link itself needs this or it will browse to the revision
-            # view instead of the file view
-            outer_breadcrumbs[-1]['suffix'] = '/files'
-            root_name = '(root)'
-            root_suffix = ''
+        # Directory Breadcrumbs
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files')
 
         # Create breadcrumb trail for the path within the branch
-        dir_parts = path.strip('/').split('/')
-        inner_breadcrumbs = []
-        for index, dir_name in enumerate(dir_parts):
-            inner_breadcrumbs.append({
-                'dir_name': dir_name,
-                'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
-            })
-
+        branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
+        
         return {
             'branch': self._branch,
             'util': util,
@@ -121,8 +100,6 @@
             'url': self._branch.context_url,
             'start_revid': start_revid,
             'fileview_active': True,
-            'outer_breadcrumbs': outer_breadcrumbs,
-            'inner_breadcrumbs': inner_breadcrumbs,
-            'root_name': root_name,
-            'root_suffix': root_suffix,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'branch_breadcrumbs': branch_breadcrumbs,
         }

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-07-22 00:40:03 +0000
+++ loggerhead/controllers/revision_ui.py	2008-08-16 21:53:52 +0000
@@ -72,6 +72,13 @@
         if side_by_side:
             h.add_side_by_side([ change ])
 
+        # Directory Breadcrumbs
+        directory_breadcrumbs = (
+            util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'changes'))
+
         return {
             'branch': self._branch,
             'revid': revid,
@@ -89,4 +96,5 @@
             'line_count': line_count,
             'line_count_limit': line_count_limit,
             'show_plain_diffs': line_count > line_count_limit,
+            'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-08-06 19:39:07 +0000
+++ loggerhead/history.py	2008-09-09 22:34:59 +0000
@@ -422,7 +422,7 @@
     def get_path(self, revid, file_id):
         if (file_id is None) or (file_id == ''):
             return ''
-        path = self._branch.repository.get_revision_inventory(revid).id2path(file_id)
+        path = self.get_inventory(revid).id2path(file_id)
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return path
@@ -430,7 +430,7 @@
     def get_file_id(self, revid, path):
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
-        return self._branch.repository.get_revision_inventory(revid).path2id(path)
+        return self.get_inventory(revid).path2id(path)
 
     def get_merge_point_list(self, revid):
         """
@@ -559,19 +559,15 @@
         trees = dict((t.get_revision_id(), t) for
                      t in self._branch.repository.revision_trees(required_trees))
         ret = []
-        self._branch.repository.lock_read()
-        try:
-            for revision in revisions:
-                if not revision.parents:
-                    old_tree = self._branch.repository.revision_tree(
-                        bzrlib.revision.NULL_REVISION)
-                else:
-                    old_tree = trees[revision.parents[0].revid]
-                tree = trees[revision.revid]
-                ret.append(tree.changes_from(old_tree))
-            return ret
-        finally:
-            self._branch.repository.unlock()
+        for revision in revisions:
+            if not revision.parents:
+                old_tree = self._branch.repository.revision_tree(
+                    bzrlib.revision.NULL_REVISION)
+            else:
+                old_tree = trees[revision.parents[0].revid]
+            tree = trees[revision.revid]
+            ret.append(tree.changes_from(old_tree))
+        return ret
 
     def _change_from_revision(self, revision):
         """

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2008-07-26 14:52:44 +0000
+++ loggerhead/templatefunctions.py	2008-08-16 04:57:38 +0000
@@ -28,6 +28,7 @@
 
 
 templatefunctions['macros'] = _pt('macros').macros
+templatefunctions['breadcrumbs'] = _pt('breadcrumbs').macros
 
 @templatefunc
 def file_change_summary(url, entry, modified_file_link):

=== modified file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/annotate.pt	2008-08-16 21:53:52 +0000
@@ -22,9 +22,9 @@
         </a>
       </tal:has-link>
       <tal:no-link condition="not: branch/branch_link">
-        <tal:branch-name replace="branch/friendly_name" />
+        <span metal:use-macro="breadcrumbs/directory"></span>
       </tal:no-link>
-      <span>: <span tal:content="path"></span> (revision <tal:revno content="change/revno"></tal:revno>)</span>
+      <span>: <span metal:use-macro="breadcrumbs/branch" /> (revision <tal:revno content="change/revno"></tal:revno>)</span>
     </h1>
 
     <div metal:fill-slot="content">

=== added file 'loggerhead/templates/breadcrumbs.pt'
--- loggerhead/templates/breadcrumbs.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/breadcrumbs.pt	2008-08-16 21:53:52 +0000
@@ -0,0 +1,22 @@
+<html>
+<body>
+<!--
+    Breadcrumbs macros
+-->
+<!--
+    These are all defined on one line in order to get the spacing looking right
+    in the output
+-->
+
+<!-- Branch breadcrumbs (for the path within the branch) -->
+<span metal:define-macro="branch" class="breadcrumb">
+    <tal:block repeat="crumb branch_breadcrumbs">/<a tal:attributes="href python:url([crumb['suffix'], change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" tal:condition="not:repeat/crumb/end" /><span tal:replace="crumb/dir_name" tal:condition="repeat/crumb/end" /></tal:block>
+</span>
+
+<!-- Directory breadcrumbs (for the path leading up to the branch) -->
+<span metal:define-macro="directory" class="breadcrumb">
+    <tal:block repeat="crumb directory_breadcrumbs"><span tal:condition="not:repeat/crumb/start">/</span><a tal:attributes="href string:/${crumb/path}${crumb/suffix}" tal:content="crumb/dir_name" /></tal:block>
+</span>
+
+</body>
+</html>

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/changelog.pt	2008-08-16 21:53:52 +0000
@@ -21,7 +21,7 @@
              </a>
            </tal:has-link>
            <tal:no-link condition="not: branch/branch_link">
-             <tal:branch-name replace="branch/friendly_name" />
+             <span metal:use-macro="breadcrumbs/directory"></span>
            </tal:no-link>
            : changes
           <tal:block condition="filter_file_id">to <span tal:content="python:history.get_path(revid, filter_file_id)" /></tal:block>

=== modified file 'loggerhead/templates/directory.pt'
--- loggerhead/templates/directory.pt	2008-08-10 02:56:11 +0000
+++ loggerhead/templates/directory.pt	2008-08-16 21:53:52 +0000
@@ -9,8 +9,7 @@
     <div id="loggerheadCont">
       <h1>
         Browsing
-        <!-- This is all one one line to get the spacing right -->
-        <span class="breadcrumb"><a href="/">(root)</a></span><span tal:repeat="crumb breadcrumbs" class="breadcrumb">/<a tal:attributes="href string:/${crumb/path}" tal:content="crumb/dir_name" /></span>
+        <span metal:use-macro="breadcrumbs/directory" />
       </h1>
 
       <div >

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2008-08-10 03:47:18 +0000
+++ loggerhead/templates/inventory.pt	2008-08-16 04:57:38 +0000
@@ -18,12 +18,10 @@
         </a>
       </tal:has-link>
       <span tal:condition="not: branch/branch_link">
-        <!-- This is all one one line to get the spacing right -->
-        <span class="breadcrumb"><a tal:attributes="href string:/${root_suffix}" tal:content="root_name" /></span><span tal:repeat="crumb outer_breadcrumbs" class="breadcrumb">/<a tal:attributes="href string:/${crumb/path}${crumb/suffix}" tal:content="crumb/dir_name" /></span>
+        <span metal:use-macro="breadcrumbs/directory" />
       </span>
       : viewing
-      <!-- This is all one one line to get the spacing right -->
-      <span tal:repeat="crumb inner_breadcrumbs" class="breadcrumb">/<a tal:attributes="href python:url(['/files', change.revno], file_id=crumb['file_id'])" tal:content="crumb/dir_name" /></span>
+      <span metal:use-macro="breadcrumbs/branch" />
       for revision <span tal:content="change/revno"/>
     </h1>
 

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/revision.pt	2008-08-16 21:53:52 +0000
@@ -23,7 +23,7 @@
             </a>
           </tal:has-link>
           <tal:no-link condition="not: branch/branch_link">
-            <tal:branch-name replace="branch/friendly_name" />
+            <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
           <span>: <tal:revno content="change/revno"></tal:revno>
         <tal:compare-to condition="python:compare_revid is not None">

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-08-06 21:25:24 +0000
+++ loggerhead/util.py	2008-08-16 21:53:52 +0000
@@ -341,6 +341,68 @@
         navigation.next_page_url = navigation.branch.context_url(
             [navigation.scan_url, next_page_revno], **params)
 
+def directory_breadcrumbs(path, is_root, view):
+    """
+    Generate breadcrumb information from the directory path given
+    
+    The path given should be a path up to any branch that is currently being
+    served
+    
+    Arguments:
+    path -- The path to convert into breadcrumbs
+    is_root -- Whether or not loggerhead is serving a branch at its root
+    view -- The type of view we are showing (files, changes etc)
+    """
+    # Is our root directory itself a branch?
+    if is_root:
+        if view == 'directory':
+            directory = 'files'
+        breadcrumbs = [{
+            'dir_name': path,
+            'path': '',
+            'suffix': view,
+        }]
+    else:
+        # Create breadcrumb trail for the path leading up to the branch
+        breadcrumbs = [{
+            'dir_name': "(root)",
+            'path': '',
+            'suffix': '',
+        }]
+        if path != '/':
+            dir_parts = path.strip('/').split('/')
+            for index, dir_name in enumerate(dir_parts):
+                breadcrumbs.append({
+                    'dir_name': dir_name,
+                    'path': '/'.join(dir_parts[:index + 1]),
+                    'suffix': '',
+                })
+            # If we are not in the directory view, the last crumb is a branch,
+            # so we need to specify a view
+            if view != 'directory':
+                breadcrumbs[-1]['suffix'] = '/' + view
+    return breadcrumbs
+
+def branch_breadcrumbs(path, inv, view):
+    """
+    Generate breadcrumb information from the branch path given
+    
+    The path given should be a path that exists within a branch
+    
+    Arguments:
+    path -- The path to convert into breadcrumbs
+    inv -- Inventory to get file information from
+    view -- The type of view we are showing (files, changes etc)
+    """
+    dir_parts = path.strip('/').split('/')
+    inner_breadcrumbs = []
+    for index, dir_name in enumerate(dir_parts):
+        inner_breadcrumbs.append({
+            'dir_name': dir_name,
+            'file_id': inv.path2id('/'.join(dir_parts[:index + 1])),
+            'suffix': '/' + view ,
+        })
+    return inner_breadcrumbs
 
 def decorator(unbound):
     def new_decorator(f):

